@@ -23,7 +23,7 @@ typedef struct _GpeClockFaceClass      GpeClockFaceClass;
 
 GtkType		gpe_clock_face_get_type (void);
 
-GtkWidget      *gpe_clock_face_new (GtkAdjustment *, GtkAdjustment *);
+GtkWidget      *gpe_clock_face_new (GtkAdjustment *, GtkAdjustment *, GtkAdjustment *);
 
 void            gpe_clock_face_set_do_grabs (GpeClockFace *clock, gboolean yes);
 
